--[[
  Copyright (c) 2021 by Plexim GmbH
  All rights reserved.

  A free license is granted to anyone to use this software for any legal
  non safety-critical purpose, including commercial applications, provided
  that:
  1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
  2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
--]] --
local Module = {}

local static = {numInstances = 0}

function Module.getBlock(globals)

  local ExtSync = require('blocks.block').getBlock(globals)
  ExtSync["instance"] = static.numInstances
  static.numInstances = static.numInstances + 1

  function ExtSync:checkMaskParameters(env)
    if Block.Mask.sync_gpio ~= Block.Mask.sync_gpio or
        type(Block.Mask.sync_gpio) ~= 'number' then
      return 'Invalid GPIO number.'
    end
  end

  function ExtSync:getDirectFeedthroughCode()
    local Require = ResourceList:new()
    local OutputSignal = StringList:new()

    local numExtSyncIn =
        globals.target.getTargetParameters()['epwms']['num_extsync_in']
    if static.numInstances > numExtSyncIn then
      if numExtSyncIn == 1 then
        return "There can be only one External Sync block."
      else
        return "There can be only %d External Sync blocks." % {numExtSyncIn}
      end
    end
    self.gpio = Block.Mask.sync_gpio
	-- resource allocation done in non direct feedthrough
    OutputSignal:append("{synco = {type = 'gpio%i', unit = %i}}" %
                            {self.instance + 1, Block.Mask.sync_gpio})

    return {
      Require = Require, 
      OutputSignal = {OutputSignal},
      UserData = {bid = ExtSync:getId()}
    }
  end

  function ExtSync:getNonDirectFeedthroughCode()
    local Require = ResourceList:new()
    if globals.target.getTargetParameters()['epwms']['type'] < 4 then
      -- dedicated sync pin - we need to claim it outright
      globals.target.allocateGpio(self.gpio, {}, Require)
    elseif not globals.target.isGpioAllocated(self.gpio) then
      -- configure pin implicitely as input
      local din_obj = self:makeBlock('din')
      din_obj:createImplicit(self.gpio, {}, Require)
    end
    return {
      Require = Require
    }
  end

  function ExtSync:finalize(f)
    return f
  end

  return ExtSync
end

return Module
